import {ALL_STATS, Stat, SuccessorOption} from "./datatypes.ts";
import {generateName, generateTitle} from "./namegen.ts";
import {choose} from "./utils.ts";

// TODO: Take a "number of improvements", use that to improve
//  each successor N times
export function generateSuccessors(nImprovements: number): SuccessorOption[] {
  let options = [];
  while (options.length < 3) {
    let option = generateSuccessor(nImprovements);
    if (!isEligible(options, option)) {
      continue;
    }
    options.push(option);
  }
  return options;
}

function isEligible(existing: SuccessorOption[], added: SuccessorOption) {
  for (let e of existing.values()) {
    if (e.name == added.name || e.title == added.title) {
      return false;
    }
  }
  return true;
}

export function generateSuccessor(nImprovements: number): SuccessorOption {
  let name = generateName();
  let title = generateTitle();
  let note = null;
  let stats: Record<Stat, number> = {
    "AGI": 10 + choose([1, 2]),
    "INT": 10 + choose([1, 2]),
    "CHA": 10 + choose([1, 2]),
    "PSI": 10 + choose([1, 2]),
  }
  let talents: Record<Stat, number> = {
    "AGI": 0,
    "INT": 0,
    "CHA": 0,
    "PSI": 0,
  }

  let improvements = [
    () => { stats[choose(ALL_STATS)] += choose([3, 4, 5, 6]); },  // avg 4.5
    () => { talents[choose(ALL_STATS)] += 1; },
  ];
  let nTotalImprovements = nImprovements + 5;
  for (let i = 0; i < nTotalImprovements; i++) {
    let improvement = improvements[Math.floor(Math.random() * improvements.length)];
    improvement();
  }

  return {name, title, note, stats, talents};
}